@@ -49,7 +49,6 @@ from torch.optim.lr_scheduler import _LRScheduler
 from torch.utils.data import DataLoader
 import torch_optimizer as torch_optimizers
 
-from bitfount.backends.pytorch.data.datafactory import _PyTorchDataFactory
 from bitfount.backends.pytorch.data.dataloaders import _PyTorchBitfountDataLoader
 from bitfount.backends.pytorch.federated.models import _PyTorchDistributedModelMixIn
 from bitfount.backends.pytorch.types import (
@@ -65,6 +64,7 @@ from bitfount.backends.pytorch.types import (
 )
 from bitfount.backends.pytorch.utils import LoggerType, _autodetect_gpu
 from bitfount.config import BITFOUNT_LOGS_DIR
+from bitfount.data.databunch import _BitfountDataBunch
 from bitfount.data.dataloader import _BitfountDataLoader
 from bitfount.data.datasets import _Dataset
 from bitfount.data.datasource import DataSource
@@ -351,10 +351,31 @@ weight-averaging).
                 self.set_pod_identifier(pod_identifiers[0])
 
         if data is not None:
-            data.load_data()
-            self._add_datasource_to_schema(
-                datasource=data, datafactory=_PyTorchDataFactory()
-            )
+            if self.datastructure.query:
+                table_schema = self.datastructure._override_schema(
+                    pod_identifier=self.pod_identifier
+                )
+                self._databunch = _BitfountDataBunch(
+                    data_structure=self.datastructure,
+                    schema=table_schema,
+                    datasource=data,
+                )
+            elif self.datastructure.table:
+                if context:
+                    table_schema = self.schema.get_table_schema(
+                        self.datastructure.get_table_name(self.pod_identifier)
+                    )
+                    self._databunch = _BitfountDataBunch(
+                        data_structure=self.datastructure,
+                        schema=table_schema,
+                        datasource=data,
+                    )
+                else:
+                    # For local training, we can add the datasource to the schema.
+                    if not data._data_is_loaded:
+                        data.load_data()
+                    self._add_datasource_to_schema(data)
+
             # Initialise model details
             if self._context != ModelContext.MODELLER:
                 self._set_dataloaders(self.batch_size)
@@ -991,7 +1012,7 @@ weight-averaging).
             if metrics:
                 self.metrics = metrics_store
 
-    def evaluate(
+    def _evaluate_local(
         self, test_dl: Optional[_BitfountDataLoader] = None, **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray]:
         """This method runs inference on the test dataloader.
@@ -1008,7 +1029,7 @@ weight-averaging).
         self._reset_test_attrs()
 
         self._pl_trainer.test(model=self, test_dataloaders=cast(DataLoader, test_dl))
-        return cast(np.ndarray, self._test_preds), cast(np.ndarray, self._test_targets)
+        return np.asarray(self._test_preds), np.asarray(self._test_targets)
 
     def predict(self, data: DataSource, **kwargs: Any) -> np.ndarray:
         """This method runs inference on the test data, returns predictions.
@@ -1024,7 +1045,7 @@ weight-averaging).
             ValueError: If no data is provided to test with.
         """
         data.load_data()
-        self._add_datasource_to_schema(data, datafactory=_PyTorchDataFactory())
+        self._add_datasource_to_schema(data)
         test_dl = self._databunch.get_test_dataloader(self.batch_size)
         if isinstance(test_dl, _BitfountDataLoader):
             logger.info(
@@ -1173,15 +1194,7 @@ class BaseTabNetModel(
         """
         self._context = context
         self._initialised = True
-
-        if data is not None:
-            data.load_data()
-            self._add_datasource_to_schema(data)
-
-        # TODO: [BIT-1152] initialise model without setting dataloaders
-        if self._context != ModelContext.MODELLER:
-            self._set_dataloaders(batch_size=self.batch_size)
-        elif self._context == ModelContext.MODELLER:
+        if self._context == ModelContext.MODELLER:
             # In a distributed setting, the Modeller needs to first initialise its
             # own model before it can be used. The pod identifier needs to be set
             # before the model is initialised so the the relevant details can be
@@ -1192,6 +1205,36 @@ class BaseTabNetModel(
             if pod_identifiers:
                 self.set_pod_identifier(pod_identifiers[0])
 
+        if data is not None:
+            if self.datastructure.query:
+                table_schema = self.datastructure._override_schema(
+                    pod_identifier=self.pod_identifier
+                )
+                self._databunch = _BitfountDataBunch(
+                    data_structure=self.datastructure,
+                    schema=table_schema,
+                    datasource=data,
+                )
+            elif self.datastructure.table:
+                if context:
+                    table_schema = self.schema.get_table_schema(
+                        self.datastructure.get_table_name(self.pod_identifier)
+                    )
+                    self._databunch = _BitfountDataBunch(
+                        data_structure=self.datastructure,
+                        schema=table_schema,
+                        datasource=data,
+                    )
+                else:
+                    # For local training, we can add the datasource to the schema.
+                    if not data._data_is_loaded:
+                        data.load_data()
+                    self._add_datasource_to_schema(data)
+
+        # TODO: [BIT-1152] initialise model without setting dataloaders
+        if self._context != ModelContext.MODELLER:
+            self._set_dataloaders(batch_size=self.batch_size)
+
         self._model = self._create_model()
         # Make sure we only use the tabular part of the data.
         train_x_df_aux = self.train_dl.get_x_dataframe()
@@ -1211,7 +1254,7 @@ class BaseTabNetModel(
             drop_last=False,
         )
 
-    def evaluate(
+    def _evaluate_local(
         self, test_dl: Optional[_BitfountDataLoader] = None, **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray]:
         """This method runs inference on the test dataloader.
