@@ -12,7 +12,6 @@ from torch import nn as nn
 from torch.optim.lr_scheduler import _LRScheduler
 from torch.utils.data import DataLoader as PyTorchDataLoader
 
-from bitfount.backends.pytorch.data.datafactory import _PyTorchDataFactory
 from bitfount.backends.pytorch.data.dataloaders import _PyTorchBitfountDataLoader
 from bitfount.backends.pytorch.federated.models import _PyTorchDistributedModelMixIn
 from bitfount.backends.pytorch.models.base_models import (
@@ -21,6 +20,7 @@ from bitfount.backends.pytorch.models.base_models import (
     _OptimizerType,
 )
 from bitfount.backends.pytorch.utils import _autodetect_gpu
+from bitfount.data.databunch import _BitfountDataBunch
 from bitfount.data.dataloader import _BitfountDataLoader
 from bitfount.data.datasource import DataSource
 from bitfount.metrics import Metric
@@ -252,20 +252,31 @@ class PyTorchBitfountModel(
         """
         self._context = context
         self._initialised = True
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, pod_identifier=self.pod_identifier, datasource=data
+        )
         if data is not None:
-            data.load_data()
-            self._add_datasource_to_schema(data, datafactory=_PyTorchDataFactory())
+            if context:
+                self._databunch = _BitfountDataBunch(
+                    data_structure=self.datastructure,
+                    schema=table_schema,
+                    datasource=data,
+                )
+            else:
+                # For local training, we can add the datasource to the schema.
+                if not data._data_is_loaded:
+                    data.load_data()
+                self._add_datasource_to_schema(data)
             if self._context != ModelContext.MODELLER:
                 self._set_dataloaders(self.batch_size)
-        self.datastructure.set_training_input_size(
-            self.schema, self.datastructure.get_table_name(self.pod_identifier)
-        )
+        self.datastructure.set_training_input_size(table_schema)
+
         if self._objective == "classification":
             # The casts here are to assuage mypy because it (incorrectly) asserts
             # that a subclass of both ClassifierMixIn and BitfountModel cannot exist.
             # We utilise a subclass of both in the tests to assure ourselves.
             if isinstance(cast(ClassifierMixIn, self), ClassifierMixIn):
-                cast(ClassifierMixIn, self).set_number_of_classes()
+                cast(ClassifierMixIn, self).set_number_of_classes(table_schema)
             else:
                 raise TypeError(
                     "Training objective is classification but this model does not "
@@ -425,7 +436,7 @@ class PyTorchBitfountModel(
             avgs[k] = float(torch.stack(v_list).mean().item())
         return avgs
 
-    def evaluate(
+    def _evaluate_local(
         self, test_dl: Optional[_BitfountDataLoader] = None, **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray]:
         """This method runs inference on the test dataloader.
@@ -462,9 +473,7 @@ class PyTorchBitfountModel(
             ValueError: If no data is provided to test with.
         """
         data.load_data()
-        self._add_datasource_to_schema(
-            datasource=data, datafactory=_PyTorchDataFactory()
-        )
+        self._add_datasource_to_schema(datasource=data)
         test_dl = self._databunch.get_test_dataloader(self.batch_size)
         if isinstance(test_dl, _BitfountDataLoader):
             logger.info(
