@@ -155,7 +155,12 @@ class PyTorchImageClassifier(PyTorchClassifierMixIn, BasePyTorchModel):
 
         Otherwise, creates model as defined by `CNNModelStructure`
         """
-        self.set_number_of_classes()
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, pod_identifier=self.pod_identifier
+        )
+
+        self.set_number_of_classes(table_schema)
+
         if isinstance(self.model_structure, NeuralNetworkPredefinedModel):
             kwargs = self.model_structure.kwargs or {}
             model = _get_torchvision_classification_model(
@@ -250,21 +255,22 @@ class PyTorchTabularClassifier(PyTorchClassifierMixIn, BasePyTorchModel):
         Takes number of continuous features and number of heads. Creates and
         returns model.
         """
-        self.set_number_of_classes()
         ignore_cols_for_training = self.datastructure.get_columns_ignored_for_training()
+
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, pod_identifier=self.pod_identifier
+        )
+        self.set_number_of_classes(table_schema)
+
         num_continuous = len(
             [
                 col
-                for col in self.schema.get_feature_names(
-                    self.datastructure.get_table_name(self.pod_identifier),
+                for col in table_schema.get_feature_names(
                     SemanticType.CONTINUOUS,
                 )
                 if col not in ignore_cols_for_training
             ]
         )
-        table_schema = self.schema.get_table_schema(
-            self.datastructure.get_table_name(self.pod_identifier)
-        )
         embedding_sizes = _calculate_embedding_sizes(
             table_schema.get_categorical_feature_sizes(ignore_cols_for_training)
         )
@@ -321,8 +327,9 @@ class PyTorchTabularClassifier(PyTorchClassifierMixIn, BasePyTorchModel):
         """
         tab, sup = cast(ImgXorTabDataSplit, data)
         ignore_cols_for_training = self.datastructure.get_columns_ignored_for_training()
-        table_schema = self.schema.get_table_schema(
-            self.datastructure.get_table_name(self.pod_identifier)
+
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, pod_identifier=self.pod_identifier
         )
         n_cat = len(
             _calculate_embedding_sizes(
@@ -332,8 +339,7 @@ class PyTorchTabularClassifier(PyTorchClassifierMixIn, BasePyTorchModel):
         n_cont = len(
             [
                 col
-                for col in self.schema.get_feature_names(
-                    self.datastructure.get_table_name(self.pod_identifier),
+                for col in table_schema.get_feature_names(
                     SemanticType.CONTINUOUS,
                 )
                 if col not in ignore_cols_for_training
@@ -450,14 +456,14 @@ class PyTorchLogisticRegressionClassifier(PyTorchClassifierMixIn, BasePyTorchMod
         self.embed_categorical_dropout = embed_categorical_dropout
 
     def _create_model(self) -> nn.Module:
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, pod_identifier=self.pod_identifier
+        )
         # Set number of output classes from datastructure
-        self.set_number_of_classes()
+        self.set_number_of_classes(table_schema)
 
         # Get details on continuous and categorical features
         ignore_cols_for_training = self.datastructure.get_columns_ignored_for_training()
-        table_schema = self.schema.get_table_schema(
-            self.datastructure.get_table_name(self.pod_identifier)
-        )
 
         if self.embed_categorical:
             # If categorical embedding requested, calculate embedding sizes
@@ -519,8 +525,8 @@ class PyTorchLogisticRegressionClassifier(PyTorchClassifierMixIn, BasePyTorchMod
 
         # Gather schema-related information
         ignore_cols_for_training = self.datastructure.get_columns_ignored_for_training()
-        table_schema = self.schema.get_table_schema(
-            self.datastructure.get_table_name(self.pod_identifier)
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, pod_identifier=self.pod_identifier
         )
 
         # Calculate the number of continuous and categorical columns
@@ -652,12 +658,12 @@ class TabNetClassifier(PyTorchClassifierMixIn, BaseTabNetModel):
 
     def _create_model(self) -> TabNetClassifier_:
         """Create model for Binary or Multiclass classification."""
-        self.set_number_of_classes()
         ignore_cols_for_training = self.datastructure.get_columns_ignored_for_training()
 
-        table_schema = self.schema.get_table_schema(
-            self.datastructure.get_table_name(self.pod_identifier)
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, pod_identifier=self.pod_identifier
         )
+        self.set_number_of_classes(table_schema)
 
         # Only consider the tabular part
         X: pd.DataFrame
@@ -669,8 +675,7 @@ class TabNetClassifier(PyTorchClassifierMixIn, BaseTabNetModel):
             i
             for i, f in enumerate(X.columns)
             if f
-            in self.schema.get_feature_names(
-                self.datastructure.get_table_name(self.pod_identifier),
+            in table_schema.get_feature_names(
                 SemanticType.CATEGORICAL,
             )
             if f not in ignore_cols_for_training
