@@ -32,8 +32,10 @@ class _BaseSource(ABC):
         self.data: Optional[_DataFrameType] = None
 
     @abstractmethod
-    def get_values(self, col_name: str, **kwargs: Any) -> Iterable[Any]:
-        """Implement this method to get distinct values from column."""
+    def get_values(
+        self, col_names: List[str], **kwargs: Any
+    ) -> Dict[str, Iterable[Any]]:
+        """Implement this method to get distinct values from list of columns."""
         raise NotImplementedError
 
     @abstractmethod
@@ -72,17 +74,22 @@ class CSVSource(_BaseSource):
         """
         return self.data
 
-    def get_values(self, col_name: str, **kwargs: Any) -> Iterable[Any]:
-        """Get distinct values from column in CSV dataset.
+    def get_values(
+        self, col_names: List[str], **kwargs: Any
+    ) -> Dict[str, Iterable[Any]]:
+        """Get distinct values from columns in CSV dataset.
 
         Args:
-            col_name: The name of the column whose distinct values should be
+            col_names: The list of the columns whose distinct values should be
                 returned.
 
+
         Returns:
-            The distinct values of the requested column as a series.
+            The distinct values of the requested column as a mapping from col name to
+            a series of distinct values.
+
         """
-        return self.data[col_name].unique()
+        return {col: self.data[col].unique() for col in col_names}
 
     def get_column(
         self, col_name: str, **kwargs: Any
@@ -124,19 +131,21 @@ class DataFrameSource(_BaseSource):
     def __init__(self, data: _DataFrameType):
         self.data: _DataFrameType = data
 
-    def get_values(self, col_name: str, **kwargs: Any) -> Iterable[Any]:
-        """Get distinct values from column in DataFrame dataset.
+    def get_values(
+        self, col_names: List[str], **kwargs: Any
+    ) -> Dict[str, Iterable[Any]]:
+        """Get distinct values from columns in DataFrame dataset.
 
         Args:
-            col_name: The name of the column whose distinct values should be
+            col_names: The list of the columns whose distinct values should be
                 returned.
-            table_name: The name of the table to which the column exists. Required
-                for multi-table databases.
+
 
         Returns:
-            The distinct values of the requested column as a series.
+            The distinct values of the requested column as a mapping from col name to
+            a series of distinct values.
         """
-        return self.data[col_name].unique()
+        return {col: self.data[col].unique() for col in col_names}
 
     def get_column(
         self, col_name: str, **kwargs: Any
@@ -239,24 +248,28 @@ class DatabaseSource(_BaseSource):
         return table_name
 
     def get_values(
-        self, col_name: str, table_name: Optional[str] = None, **kwargs: Any
-    ) -> Iterable[Any]:
-        """Get distinct values from column in Database dataset.
+        self, col_names: List[str], table_name: Optional[str] = None, **kwargs: Any
+    ) -> Dict[str, Iterable[Any]]:
+        """Get distinct values from columns in Database dataset.
 
         Args:
-            col_name: The name of the column whose distinct values should be
+            col_names: The list of the columns whose distinct values should be
                 returned.
             table_name: The name of the table to which the column exists. Required
                 for multi-table databases.
 
+
         Returns:
-            The distinct values of the requested column as a series.
+            The distinct values of the requested column as a mapping from col name to
+            a series of distinct values.
         """
-        results: Iterable[Any]
         metadata = MetaData(self.con)
+        output = {}
         if self.query is not None:
-            with Session(self.con) as session:
-                results = session.execute(self.query).columns(col_name).unique()
+            # TODO: [BIT-1595] change to load memory using sqlalchemy FrozenResult
+            self.data = pd.read_sql(self.query, con=self.con)
+            for col_name in col_names:
+                output[col_name] = self.data[col_name].unique()
         else:
             table_name = self._validate_table_name(table_name)
 
@@ -268,9 +281,12 @@ class DatabaseSource(_BaseSource):
                 autoload_with=self.con,
             )
             with Session(self.con) as session:
-                results = session.query(table.columns[col_name]).distinct()
-        values = np.array([v for v, in results])
-        return values
+                for col_name in col_names:
+                    values = np.array(
+                        [v for v, in session.query(table.columns[col_name]).distinct()]
+                    )
+                    output[col_name] = values
+        return output
 
     def get_column(
         self, col_name: str, **kwargs: Any
@@ -320,12 +336,15 @@ class DatabaseSource(_BaseSource):
             A DataFrame-type object which contains the data.
         """
         sql_query = kwargs.get("sql_query")
+        table_name = kwargs.get("table_name")
         data: Optional[_DataFrameType] = None
         if self.db_conn.multi_table:
             # If data is multi-table, `data.table_names` can't be None
             assert self.db_conn.table_names is not None  # nosec
             if sql_query:
                 data = self._bf.read_sql_query(sql=sql_query, con=self.db_conn.con)
+            elif table_name:
+                data = self._bf.read_sql_table(table_name=table_name, con=self.con)
         elif self.db_conn.query:
             data = self._bf.read_sql_query(sql=self.db_conn.query, con=self.db_conn.con)
         else:
@@ -397,8 +416,6 @@ class DataSource:
         data_ref: The reference of the data to load.
         data: A Dataframe-type object which contains the data.
         seed: Random number seed. Used for setting random seed for all libraries.
-        auto_tidy: Whether to tidy the data automatically (cleaning and
-            normalizing).
         train_idxs: A numpy array containing the indices of the data which
             will be used for training.
         validation_idxs: A numpy array containing the indices of the data which
@@ -520,7 +537,9 @@ class DataSource:
                 self.data[column_name], modifier_dict
             )
 
-    def _get_data(self, sql_query: Optional[str] = None) -> Optional[_DataFrameType]:
+    def _get_data(
+        self, sql_query: Optional[str] = None, table_name: Optional[str] = None
+    ) -> Optional[_DataFrameType]:
         """Loads and returns data.
 
         Args:
@@ -529,7 +548,7 @@ class DataSource:
         Returns:
             A DataFrame-type object which contains the data.
         """
-        data = self.loader.get_data(sql_query=sql_query)
+        data = self.loader.get_data(sql_query=sql_query, table_name=table_name)
         return data
 
     def get_features(self) -> List[str]:
@@ -574,20 +593,21 @@ class DataSource:
         return dtypes
 
     def get_values(
-        self, col_name: str, table_name: Optional[str] = None
-    ) -> Iterable[Any]:
-        """Get distinct values from column in data.
+        self, col_names: List[str], table_name: Optional[str] = None
+    ) -> Dict[str, Iterable[Any]]:
+        """Get distinct values from columns in data.
 
         Args:
-            col_name: The name of the column whose distinct values should be
+            col_names: The list of the column whose distinct values should be
                 returned.
             table_name: The name of the table to which the column exists. Required
                 for multi-table databases.
 
         Returns:
-            The distinct values of the requested column as a series.
+            The distinct values of the requested column as a mapping from col name to
+            a series of distinct values.
         """
-        return self.loader.get_values(col_name=col_name, table_name=table_name)
+        return self.loader.get_values(col_names=col_names, table_name=table_name)
 
     def get_column(
         self, col_name: str, table_name: Optional[str] = None
@@ -608,7 +628,9 @@ class DataSource:
                 column = self._modify_column(column, modifier_dict)
         return column
 
-    def load_data(self, sql_query: Optional[str] = None) -> None:
+    def load_data(
+        self, sql_query: Optional[str] = None, table_name: Optional[str] = None
+    ) -> None:
         """Load the data for the datasource.
 
         This method is idempotent so it can be called multiple times without
@@ -618,7 +640,7 @@ class DataSource:
             TypeError: If data format is not supported.#
         """
         if not self._data_is_loaded:
-            data = self._get_data(sql_query)
+            data = self._get_data(sql_query=sql_query, table_name=table_name)
             if not self.multi_table:
                 assert data is not None  # nosec
             if data is not None:
