@@ -265,6 +265,11 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
         semantic_types = self._combine_existing_stypes_with_forced_stypes(
             inferred_semantic_types, force_stype, table_dtypes
         )
+        if SemanticType.CATEGORICAL in semantic_types.keys():
+            values = datasource.get_values(
+                semantic_types[SemanticType.CATEGORICAL], self.name
+            )
+
         for stype, features in semantic_types.items():
             # Sort the list of features.
             # This ensures they are added in deterministic order.
@@ -293,7 +298,7 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
                     self._add_categorical_feature(
                         name=feature_name,
                         dtype=dtype,
-                        values=datasource.get_values(feature_name, self.name),
+                        values=values[feature_name],
                         description=description,
                     )
                 elif stype == SemanticType.IMAGE:
@@ -485,7 +490,6 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
             raise ValueError("No categorical features.")
         elif var not in self.features["categorical"]:
             raise ValueError(f"{var} feature not found in categorical features.")
-
         return self.features["categorical"][var].encoder.size
 
     def get_categorical_feature_sizes(
