@@ -32,6 +32,12 @@ class PodRegistrationError(BitfountError):
     pass
 
 
+class PodNameError(BitfountError):
+    """Error related to given Pod name."""
+
+    pass
+
+
 class SecureShareError(BitfountError):
     """Error related to SecureShare processes."""
 
