@@ -32,8 +32,11 @@ from bitfount.federated.authorisation_checkers import (
     _SAMLAuthorisation,
     check_identity_verification_method,
 )
-from bitfount.federated.exceptions import PodRegistrationError
-from bitfount.federated.helper import _create_and_connect_pod_mailbox
+from bitfount.federated.exceptions import PodNameError, PodRegistrationError
+from bitfount.federated.helper import (
+    _check_and_update_pod_ids,
+    _create_and_connect_pod_mailbox,
+)
 from bitfount.federated.logging import _get_federated_logger
 from bitfount.federated.pod_keys_setup import PodKeys, _get_pod_keys
 from bitfount.federated.pod_response_message import _PodResponseMessage
@@ -52,7 +55,6 @@ from bitfount.federated.transport.worker_transport import (
     _WorkerMailbox,
 )
 from bitfount.federated.types import AggregatorType, _PodResponseType
-from bitfount.federated.utils import _validate_pod_identifiers
 from bitfount.federated.worker import _Worker
 from bitfount.hub.api import BitfountAM, BitfountHub, PodPublicMetadata
 from bitfount.hub.authentication_flow import (
@@ -64,7 +66,11 @@ from bitfount.hub.helper import (
     _create_bitfounthub,
     _get_pod_public_keys,
 )
-from bitfount.runners.config_schemas import PodDataConfig, PodDetailsConfig
+from bitfount.runners.config_schemas import (
+    POD_NAME_REGEX,
+    PodDataConfig,
+    PodDetailsConfig,
+)
 from bitfount.transformations.dataset_operations import (
     CleanDataTransformation,
     NormalizeDataTransformation,
@@ -202,7 +208,7 @@ class Pod:
         self.pod_identifier = f"{self._session.username}/{self.name}"
         if approved_pods is None:
             approved_pods = []
-        _validate_pod_identifiers([*approved_pods, self.pod_identifier])
+        _check_and_update_pod_ids([*approved_pods, self.pod_identifier], self._hub)
         self.approved_pods = approved_pods
 
         self._pod_dp = pod_dp
@@ -218,6 +224,22 @@ class Pod:
         # Marker for when initialization is complete
         self._initialised: bool = False
 
+    @property
+    def name(self) -> str:
+        """Pod name property."""
+        return self._name
+
+    @name.setter
+    def name(self, name: str) -> None:
+        """Validate Pod's name matches POD_NAME_REGEX."""
+        if _name := POD_NAME_REGEX.fullmatch(name):
+            self._name = _name.string
+        else:
+            raise PodNameError(
+                f"Invalid Pod name: {name}. "
+                f"Pod names must match: {POD_NAME_REGEX.pattern}"
+            )
+
     def _setup_schema(
         self,
         data_config: PodDataConfig,
