@@ -27,7 +27,6 @@ from typing import (
     cast,
 )
 
-from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
 import desert
 from marshmallow import Schema as MarshmallowSchema
 from marshmallow import fields, post_load
@@ -35,11 +34,10 @@ import numpy as np
 
 from bitfount.config import BITFOUNT_LOGS_DIR
 from bitfount.data.databunch import _BitfountDataBunch
-from bitfount.data.datafactory import _BasicDataFactory, _DataFactory
 from bitfount.data.dataloader import _BitfountDataLoader
 from bitfount.data.datasource import DataSource
 from bitfount.data.datastructure import DataStructure
-from bitfount.data.schema import BitfountSchema
+from bitfount.data.schema import BitfountSchema, TableSchema
 from bitfount.types import DistributedModelProtocol, _JSONDict
 from bitfount.utils import _add_this_to_list, seed_all
 
@@ -161,15 +159,17 @@ class _BaseModel(ABC, Generic[ModelType]):
     def _add_datasource_to_schema(
         self,
         datasource: DataSource,
-        datafactory: Optional[_DataFactory] = None,
     ) -> None:
-        """Adds the datasource to the schema and sets the `_databunch` attribute."""
+        """Adds the datasource to the schema.
+
+        Should be only used for local training.
+        """
         # self.datastructure.apply_transformations(datasource)
         # TODO: [BIT-1167] Once we add the transformations we can force the target
         #  to be categorical in the transformations as this can be specified directly
-        if datafactory is None:
-            datafactory = _BasicDataFactory()
         if self.datastructure.image_cols:
+            if not datasource._data_is_loaded:
+                datasource.load_data()
             for image in self.datastructure.image_cols:
                 if image not in datasource.data.columns:
                     raise ValueError(f"Could not find {image} in dataset columns")
@@ -193,14 +193,10 @@ class _BaseModel(ABC, Generic[ModelType]):
             force_stypes={table_name: self.datastructure._force_stype},
             table_name=table_name,
         )
-        datasource = self.datastructure.apply_dataset_transformations(datasource)
-
-        # Create the BitfountDataBunch that will be used in training
         self._databunch = _BitfountDataBunch(
             data_structure=self.datastructure,
             schema=self.schema.get_table_schema(table_name),
             datasource=datasource,
-            data_factory=datafactory,
         )
 
     def initialise_model(
@@ -218,11 +214,8 @@ class _BaseModel(ABC, Generic[ModelType]):
                 there is no difference between modeller and worker. Defaults to None.
         """
         self._initialised = True
-        if data:
-            data.load_data()
-            self._add_datasource_to_schema(
-                datasource=data,
-            )
+        if context is None and data is not None:
+            self._add_datasource_to_schema(datasource=data)
 
     @property
     def initialised(self) -> bool:
@@ -246,14 +239,16 @@ class _BaseModel(ABC, Generic[ModelType]):
     def evaluate(
         self,
         test_dl: Optional[_BitfountDataLoader] = None,
+        pod_identifiers: Optional[List[str]] = None,
         **kwargs: Any,
-    ) -> Tuple[np.ndarray, np.ndarray]:
+    ) -> Union[Tuple[np.ndarray, np.ndarray], Dict[str, float]]:
         """Implement this method to perform inference on the test set.
 
         Args:
             test_dl: Optional `BitfountDataLoader` object containing test data. If this
                 is not provided, the test set from the `DataSource` used to train the
                 model should be used if present.
+            pod_identifiers: A list of pod identifiers to use for remote evaluation.
 
         Returns:
             A tuple of numpy arrays containing the predicted and actual values.
@@ -279,20 +274,18 @@ class _BaseModel(ABC, Generic[ModelType]):
         data: Optional[DataSource] = None,
         metrics: Optional[Dict[str, Metric]] = None,
         pod_identifiers: Optional[List[str]] = None,
-        private_key_or_file: Optional[Union[RSAPrivateKey, Path]] = None,
         **kwargs: Any,
     ) -> Optional[Dict[str, str]]:
         """Must be implemented to fit the model.
 
+        Must call `initialise_model()` within the method if the model needs to be
+        initialised.
+
         Args:
             data: The data used for local model training.
             metrics: A dictionary of metrics to use for validation.
             pod_identifiers: A list of pod identifiers to use for federated training.
-            private_key_or_file: A private key or path to private key file if doing
-                federated training.
 
-        Must call `initialise_model()` within the method if the model needs to be
-        initialised.
         """
         raise NotImplementedError
 
@@ -348,7 +341,7 @@ class ClassifierMixIn:
             self.n_classes = int(kwargs["n_classes"])
         self._objective = "classification"
 
-    def set_number_of_classes(self) -> None:
+    def set_number_of_classes(self, schema: TableSchema) -> None:
         """Sets the target number of classes for the classifier.
 
         If the data is a multi-label problem, the number of classes is set to the number
@@ -365,19 +358,10 @@ class ClassifierMixIn:
         elif self.datastructure.target is not None:
             # If the model is distributed, then we need to get the table name that
             # the Modeller has specified for this particular Pod.
-            pod_identifier: Optional[str] = None
-            if hasattr(self, "pod_identifier"):
-                # Assuring mypy that the attribute exists
-                assert isinstance(self, DistributedModelProtocol)  # nosec
-                pod_identifier = self.pod_identifier
-            table_name = self.datastructure.get_table_name(pod_identifier)
-
             self.n_classes = (
                 len(self.datastructure.target)
                 if self.multilabel
-                else self.schema.get_categorical_feature_size(
-                    table_name, self.datastructure.target
-                )
+                else schema.get_categorical_feature_size(self.datastructure.target)
             )
         else:
             raise ValueError(
@@ -388,12 +372,12 @@ class ClassifierMixIn:
     def _add_datasource_to_schema(
         self,
         datasource: DataSource,
-        datafactory: Optional[_DataFactory] = None,
     ) -> None:
-        """Adds the datasource to the schema and sets the `_databunch` attribute."""
+        """Adds the datasource to the schema.
+
+        Used for local training.
+        """
         # Add the datasource features to the datastructure.
-        if datafactory is None:
-            datafactory = _BasicDataFactory()
         force_categorical: List[str] = []
         force_categorical = _add_this_to_list(
             self.datastructure.target, force_categorical
@@ -413,31 +397,20 @@ class ClassifierMixIn:
                 force_categorical, self.datastructure._force_stype["categorical"]
             )
 
-        # If the model is distributed, then we need to get the table name that
-        # the Modeller has specified for this particular Pod.
-        pod_identifier: Optional[str] = None
-        if hasattr(self, "pod_identifier"):
-            # Assuring mypy that the attribute exists
-            assert isinstance(self, DistributedModelProtocol)  # nosec
-            pod_identifier = self.pod_identifier
-        table_name = self.datastructure.get_table_name(pod_identifier)
-
         # This intentionally does not support multi-table datasources because a
         # multi-table datasource should have been reduced to a single table by this
         # point
+        table_name = self.datastructure.get_table_name()
         self.schema.add_datasource_tables(
             datasource,
             ignore_cols={table_name: self.datastructure.ignore_cols},
             force_stypes={table_name: self.datastructure._force_stype},
             table_name=table_name,
         )
-        datasource = self.datastructure.apply_dataset_transformations(datasource)
-        # Create the BitfountDataBunch that will be used in training
         self._databunch = _BitfountDataBunch(
             data_structure=self.datastructure,
             schema=self.schema.get_table_schema(table_name),
             datasource=datasource,
-            data_factory=datafactory,
         )
 
     class _Schema(MarshmallowSchema):
