@@ -108,7 +108,7 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
             self._model = pickle.load(f)  # nosec
 
     def evaluate(
-        self, test_dl: Optional[_BitfountDataLoader] = None, **kwargs: Any
+        self, test_dl: Optional[_BitfountDataLoader] = None, *args: Any, **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray]:
         """Perform inference on test set and return predictions and targets.
 
@@ -156,10 +156,15 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
         :::
         """
         logger.info("Fitting logistic regression classifier.")
+        if data:
+            data.load_data()
+        table_schema = self.datastructure.get_table_schema(
+            schema=self.schema, datasource=data
+        )
 
         if not self.initialised:
-            self.initialise_model(data=data)
-        self.set_number_of_classes()
+            self.initialise_model(data)
+        self.set_number_of_classes(table_schema)
         self._set_dataloaders()
         model = sklearnLogisticRegression(
             C=self.inverse_regularisation,
@@ -168,7 +173,6 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
             solver=self.model_type,
             verbose=self.verbose,
         )
-
         model.fit(
             self.train_dl.get_x_dataframe(),
             self.train_dl.get_y_dataframe()[self._databunch.target].to_numpy(),
@@ -498,7 +502,7 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
         self._model: RegBoostRegressor._ModelTree = model
 
     def evaluate(
-        self, test_dl: Optional[_BitfountDataLoader] = None, **kwargs: Any
+        self, test_dl: Optional[_BitfountDataLoader] = None, *args: Any, **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray]:
         """Perform inference on test set and return predictions and targets.
 
