@@ -4,11 +4,12 @@ from __future__ import annotations
 from dataclasses import dataclass
 import logging
 from pathlib import Path
+import re
 import typing
 from typing import Dict, List, Mapping, MutableMapping, Optional, Union
 
 import desert
-from marshmallow import ValidationError, fields
+from marshmallow import ValidationError, fields, validate
 from marshmallow.validate import OneOf
 from marshmallow_union import Union as M_Union
 
@@ -17,7 +18,11 @@ from bitfount.config import (
     _STAGING_ENVIRONMENT,
     _get_environment,
 )
-from bitfount.data.types import DataPathModifiers, _SemanticTypeValue
+from bitfount.data.types import (
+    DataPathModifiers,
+    SchemaOverrideMapping,
+    _SemanticTypeValue,
+)
 from bitfount.data.utils import DatabaseConnection
 from bitfount.federated.authorisation_checkers import (
     DEFAULT_IDENTITY_VERIFICATION_METHOD,
@@ -53,6 +58,12 @@ from bitfount.types import _JSONDict
 
 logger = logging.getLogger(__name__)
 
+# POD_NAME_REGEX = re.compile(r"[a-z\d]+(-[a-z\d]+)*")
+# USERNAME_REGEX = re.compile(r"[a-z\d](?:[a-z\d]|-(?=[a-z\d])){0,38}")
+# TODO: [BIT-1493] revert to regex above to disallow underscores
+POD_NAME_REGEX = re.compile(r"[a-z\d]+((-|_)[a-z\d]+)*")
+USERNAME_REGEX = re.compile(r"[a-z\d](?:[a-z\d]|(-|_)(?=[a-z\d])){0,38}")
+
 # MYPY ISSUES
 # Due to the use of desert.field to create non-standard fields in these dataclasses,
 # mypy complains about incompatible type assignment. This is a known bug in desert
@@ -108,7 +119,10 @@ class HubConfig:
 class ModellerUserConfig:
     """Configuration for the modeller."""
 
-    username: str = _DEFAULT_USER_DIRECTORY
+    username: str = desert.field(
+        fields.String(validate=validate.Regexp(USERNAME_REGEX)),
+        default=_DEFAULT_USER_DIRECTORY,
+    )  # type: ignore[assignment] # Reason: see top of file
 
     identity_verification_method: str = desert.field(
         fields.String(validate=OneOf(IDENTITY_VERIFICATION_METHODS)),
@@ -310,10 +324,12 @@ class TaskConfig:
 class PodConfig:
     """Full configuration for the pod."""
 
-    pod_name: str
     data: Union[DatabaseConfig, PathConfig]
     data_config: PodDataConfig
     pod_details: PodDetailsConfig
+    pod_name: str = fields.String(
+        validate=validate.Regexp(POD_NAME_REGEX)
+    )  # type: ignore[assignment] # Reason: see top of file
     schema: Optional[Path] = desert.field(
         fields.Function(deserialize=_deserialize_path), default=None
     )  # type: ignore[assignment] # Reason: see top of file
@@ -330,7 +346,10 @@ class PodConfig:
     )  # type: ignore[assignment] # Reason: see top of file
     differential_privacy: Optional[DPPodConfig] = None
     other_pods: Optional[List[str]] = None
-    username: str = _DEFAULT_USER_DIRECTORY
+    username: str = desert.field(
+        fields.String(validate=validate.Regexp(USERNAME_REGEX)),
+        default=_DEFAULT_USER_DIRECTORY,
+    )  # type: ignore[assignment] # Reason: see top of file
 
     def __post_init__(self) -> None:
         environment = _get_environment()
@@ -416,8 +435,9 @@ class PodDataConfig:
             constructor.
         data_split: The data split configuration. This is passed to the `DataSource`.
         koalas: Whether to use Koalas. This is passed to the `DataSource`.
-        auto_tidy: Whether to automatically tidy the data. This is used by the `Pod`.
-
+        auto_tidy: Whether to automatically tidy the data. This is used by the
+            `Pod` and will result in removal of NaNs and normalisation of numeric
+            values. Defaults to False.
     """
 
     # There is a bug in desert (https://github.com/python-desert/desert/issues/122)
@@ -462,7 +482,7 @@ class PodDataConfig:
         default_factory=DataSplitConfig,
     )  # type: ignore[assignment] # Reason: see top of file
     koalas: bool = False
-    auto_tidy: bool = True
+    auto_tidy: bool = False
 
 
 @dataclass
@@ -514,11 +534,51 @@ class DataStructureTransformConfig:
     batch: Optional[List[Dict[str, _JSONDict]]] = None
 
 
+@dataclass
+class DataStructureTableConfig:
+    """Configuration for the datastructure table arguments."""
+
+    table: Optional[Union[str, Dict[str, str]]] = desert.field(
+        M_Union(
+            [
+                fields.String(),
+                fields.Dict(keys=fields.String(), values=fields.String(), default=None),
+            ],
+            default=None,
+        ),
+        default=None,
+    )  # type: ignore[assignment] # Reason: see top of file
+    query: Optional[Union[str, Dict[str, str]]] = desert.field(
+        M_Union(
+            [
+                fields.String(),
+                fields.Dict(keys=fields.String(), values=fields.String(), default=None),
+            ],
+            default=None,
+        ),
+        default=None,
+    )  # type: ignore[assignment] # Reason: see top of file
+    schema_types_override: Optional[
+        Union[SchemaOverrideMapping, Mapping[str, SchemaOverrideMapping]]
+    ] = desert.field(
+        fields.Dict(
+            keys=fields.String(),
+            values=fields.List(
+                M_Union([fields.String(default=None), fields.Dict(default=None)])
+            ),
+            default=None,
+        ),
+        default=None,
+    )  # type: ignore[assignment] # Reason: see top of file
+
+
 @dataclass
 class DataStructureConfig:
     """Configuration for the modeller schema and dataset options."""
 
-    table: Union[str, Dict[str, str]]
+    table_config: DataStructureTableConfig = desert.field(
+        fields.Nested(desert.schema_class(DataStructureTableConfig)),
+    )  # type: ignore[assignment] # Reason: see top of file
     assign: DataStructureAssignConfig = desert.field(
         fields.Nested(desert.schema_class(DataStructureAssignConfig)),
     )  # type: ignore[assignment] # Reason: see top of file
