@@ -30,6 +30,7 @@ from pytorch_lightning.loggers import (
 from pytorch_lightning.loggers import CSVLogger
 from pytorch_lightning.loggers import LightningLoggerBase as LightningLoggers
 from sklearn.metrics import roc_auc_score
+import sqlalchemy
 import torch
 from torch import nn as nn
 from torch.nn import CrossEntropyLoss, Linear, ReLU
@@ -49,6 +50,7 @@ from bitfount.data.datasplitters import PercentageSplitter
 from bitfount.data.datastructure import DataStructure
 from bitfount.data.schema import BitfountSchema, TableSchema
 from bitfount.data.types import _SemanticTypeValue
+from bitfount.data.utils import DatabaseConnection
 from bitfount.federated.modeller import Modeller
 from bitfount.federated.privacy.differential import DPModellerConfig
 from bitfount.metrics import (
@@ -77,6 +79,7 @@ from tests.utils.helper import (
     create_dataset,
     create_datasource,
     create_datastructure,
+    create_query_datastructure,
     create_schema,
     get_datastructure_and_datasource,
     get_info_logs,
@@ -134,6 +137,12 @@ def mock_databunch(mocker: MockerFixture) -> None:
     mocker.patch.object(_BitfountDataBunch, "create_datasets", autospec=True)
 
 
+@pytest.fixture
+def query_datastructure() -> DataStructure:
+    """Fixture for datastructure containing query."""
+    return create_query_datastructure()
+
+
 @backend_test
 class TestPyTorchModel:
     """Test BasePyTorchModel class and all subclasses."""
@@ -249,6 +258,24 @@ class TestPyTorchModel:
         model.initialise_model(context=ModelContext.MODELLER)
         assert model.n_classes == 2
 
+    @unit_test
+    def test_model_init_with_db_query(
+        self,
+        db_session: sqlalchemy.engine.base.Engine,
+        query_datastructure: DataStructure,
+        schema: BitfountSchema,
+    ) -> None:
+        """Tests that model is initialised with a query datastructure."""
+        model = PyTorchTabularClassifier(
+            datastructure=query_datastructure, schema=schema, steps=2
+        )
+        db_datasource = DataSource(data_ref=DatabaseConnection(db_session))
+        expected_schema = query_datastructure._override_schema(db_datasource)
+        model.initialise_model(db_datasource)
+        assert model._databunch is not None
+        assert model.n_classes == 2
+        assert model._databunch.schema == expected_schema
+
     @unit_test
     @pytest.mark.parametrize(
         "epochs, steps, value_error",
@@ -454,13 +481,15 @@ class TestPyTorchModel:
         inference_datasource = DataSource(datasource.data, ignore_cols=["TARGET"])
         inference_datastructure = DataStructure(table=TABLE_NAME)
         inference_datastructure.target = None
+        inference_schema = BitfountSchema(inference_datasource, table_name=TABLE_NAME)
         inference_network = PyTorchTabularClassifier(
             datastructure=inference_datastructure,
-            schema=BitfountSchema(inference_datasource, table_name=TABLE_NAME),
+            schema=inference_schema,
             epochs=1,
             n_classes=2,
         )
-        inference_network.set_number_of_classes()
+        table_schema = inference_schema.get_table_schema(TABLE_NAME)
+        inference_network.set_number_of_classes(table_schema)
 
     @unit_test
     def test_no_classes_raises_error(self, datasource: DataSource) -> None:
@@ -821,7 +850,9 @@ class TestPyTorchModel:
         )
         neural_network.fit(datasource)
         neural_network.deserialize(tmp_path / SERIALIZED_MODEL_NAME)
-        neural_network.evaluate(neural_network.test_dl)
+        preds, targs = neural_network.evaluate(neural_network.test_dl)
+        assert isinstance(preds, np.ndarray)
+        assert isinstance(targs, np.ndarray)
 
     @unit_test
     def test_tabular_invalid_model_structure(
@@ -1188,7 +1219,7 @@ class TestPyTorchModel:
     ) -> None:
         """Tests private _fit_federated method.
 
-        Checks that DistributedModelMixIn._fit_federated is helper method creates
+        Checks that DistributedModelMixIn._fit_federated helper method creates
         correct instances and runs the modeller correctly.
         """
         # Create model to test
@@ -1217,13 +1248,15 @@ class TestPyTorchModel:
         mock_modeller_run_method.assert_called_once_with(pod_identifiers)
 
     @unit_test
-    def test__fit_federated_expands_datastructure_pod_identifiers(
+    @pytest.mark.parametrize("evaluate", [True, False], ids=["evaluate", "no_evaluate"])
+    def test__fit_evaluate_federated_expands_datastructure_pod_identifiers(
         self,
         datastructure: DataStructure,
         mock_bitfount_session: Mock,
         mocker: MockerFixture,
+        evaluate: bool,
     ) -> None:
-        """Tests private _fit_federated method.
+        """Tests private `_fit_federated` and `_evaluate_federated` methods.
 
         Checks that the method expands pod identifiers in the datastructure if they are
         not provided in full.
@@ -1244,10 +1277,16 @@ class TestPyTorchModel:
 
         # Run method
         pod_identifiers = ["bitfount/adult", "bitfount/adult2"]
-        model._fit_federated(
-            pod_identifiers=pod_identifiers,
-            private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
-        )
+        if evaluate:
+            model._evaluate_federated(
+                pod_identifiers=pod_identifiers,
+                private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
+            )
+        else:
+            model._fit_federated(
+                pod_identifiers=pod_identifiers,
+                private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
+            )
 
         # Check run method was called correctly
         # TODO: [BIT-983] Should this check that the Modeller was instantiated
@@ -1284,6 +1323,78 @@ class TestPyTorchModel:
             pod_identifiers=pod_identifiers, private_key_or_file=private_key_or_file
         )
 
+    @unit_test
+    def test_evaluate_method_with_remote_arguments(
+        self, datastructure: DataStructure, mocker: MockerFixture
+    ) -> None:
+        """Test evaluate method with remote arguments detected correctly."""
+        mock_evaluate_federated_method = mocker.patch(
+            "bitfount.federated.mixins._DistributedModelMixIn._evaluate_federated",
+        )
+        model = PyTorchTabularClassifier(
+            datastructure=datastructure,
+            schema=BitfountSchema(),
+            epochs=1,
+            batch_size=16,
+        )
+        pod_identifiers = ["bitfount/adult", "bitfount/adult2"]
+        private_key_or_file = TEST_SECURITY_FILES / "test_private.testkey"
+        model.evaluate(
+            pod_identifiers=pod_identifiers,
+            private_key_or_file=private_key_or_file,
+        )
+        mock_evaluate_federated_method.assert_called_once_with(
+            pod_identifiers=pod_identifiers, private_key_or_file=private_key_or_file
+        )
+
+    @unit_test
+    def test__evaluate_federated(
+        self,
+        datastructure: DataStructure,
+        mock_bitfount_session: Mock,
+        mocker: MockerFixture,
+    ) -> None:
+        """Tests private _evaluate_federated method.
+
+        Checks that DistributedModelMixIn._evaluate_federated helper method creates
+        correct instances and runs the modeller correctly.
+        """
+        # Create model to test
+        model = PyTorchTabularClassifier(
+            datastructure=datastructure,
+            schema=BitfountSchema(),
+            epochs=1,
+            batch_size=16,
+        )
+
+        # Patch out the modeller's run method as we only care how it is called
+        # from _fit_federated.
+
+        mock_modeller_run_method = mocker.patch.object(Modeller, "run")
+
+        # Run method
+        pod_identifiers = ["bitfount/adult", "bitfount/adult2"]
+        model._evaluate_federated(
+            pod_identifiers=pod_identifiers,
+            private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
+        )
+
+        # Check run method was called correctly
+        # TODO: [BIT-983] Should this check that the Modeller was instantiated
+        #       correctly? Related to whether we should mock out the helper calls.
+        mock_modeller_run_method.assert_called_once_with(pod_identifiers)
+
+    @unit_test
+    def test_set_pod_identifier(self, datastructure: DataStructure) -> None:
+        """Tests `set_pod_identifier` method."""
+        model = PyTorchTabularClassifier(
+            datastructure=datastructure,
+            schema=BitfountSchema(),
+            epochs=1,
+        )
+        model.set_pod_identifier("test_pod")
+        assert model.pod_identifier == "test_pod"
+
 
 @backend_test
 @unit_test
@@ -1696,6 +1807,8 @@ class TestTabNetClassifier:
         )
         model.fit(datasource)
         preds, targs = model.evaluate()
+        assert isinstance(preds, np.ndarray)
+        assert isinstance(targs, np.ndarray)
         metrics = MetricCollection.create_from_model(model)
         results = metrics.compute(targs, preds)
         assert isinstance(results, dict)
@@ -1719,6 +1832,67 @@ class TestTabNetClassifier:
                 virtual_batch_size=64,
             )
 
+    @unit_test
+    def test_tabnet_init_db_query(
+        self,
+        db_session: sqlalchemy.engine.base.Engine,
+        query_datastructure: DataStructure,
+        schema: BitfountSchema,
+    ) -> None:
+        """Tests that model is initialised with a query datastructure."""
+        ms = NeuralNetworkPredefinedModel("TabNet")
+        model = TabNetClassifier(
+            model_structure=ms,
+            datastructure=query_datastructure,
+            schema=schema,
+            epochs=3,
+            batch_size=64,
+            virtual_batch_size=16,
+            optimizer=Optimizer("AdaBelief", {"lr": 0.01}),
+            seed=43,
+        )
+        db_datasource = DataSource(data_ref=DatabaseConnection(db_session))
+        expected_schema = query_datastructure._override_schema(db_datasource)
+        model.initialise_model(db_datasource)
+        assert model._databunch is not None
+        assert model.n_classes == 2
+        assert model._databunch.schema == expected_schema
+
+    @unit_test
+    def test_pod_id_set_for_modeller(
+        self,
+        schema: BitfountSchema,
+        datasource: DataSource,
+        mocker: MockerFixture,
+    ) -> None:
+        """Tests that pod_identifier is set for the modeller."""
+        ms = NeuralNetworkPredefinedModel("TabNet")
+        datastructure = DataStructure(table={"pod_id": "test_table"}, target="TARGET")
+        model = TabNetClassifier(
+            model_structure=ms,
+            datastructure=datastructure,
+            schema=schema,
+            epochs=3,
+            batch_size=64,
+            virtual_batch_size=16,
+            optimizer=Optimizer("AdaBelief", {"lr": 0.01}),
+            seed=43,
+        )
+        mock_table_schema = mocker.patch.object(
+            BitfountSchema,
+            "get_table_schema",
+            return_value=schema.get_table_schema("test_table"),
+        )
+        model.train_dl = Mock(autospec=True)
+        model.train_dl.get_y_dataframe.return_value = create_dataset(
+            classification=True
+        )
+        mocker.patch.object(TabNetClassifier, "_create_model")
+        mocker.patch.object(TabNetClassifier, "fit")
+        model.initialise_model(context=ModelContext.MODELLER, data=datasource)
+        assert model.pod_identifier == "pod_id"
+        mock_table_schema.assert_called_once()
+
     @unit_test
     def test_virtual_batch_same_as_batch_size_is_accepted(
         self, datastructure: DataStructure
@@ -1755,6 +1929,8 @@ class TestTabNetClassifier:
         )
         model.fit(datasource, metrics={"auc": roc_auc_score})
         preds, targs = model.evaluate()
+        assert isinstance(preds, np.ndarray)
+        assert isinstance(targs, np.ndarray)
         metrics = MetricCollection.create_from_model(model)
         results = metrics.compute(targs, preds)
         assert isinstance(results, dict)
@@ -1907,6 +2083,8 @@ class TestTabNetClassifier:
         )
         model.fit(datasource)
         preds, targs = model.evaluate()
+        assert isinstance(preds, np.ndarray)
+        assert isinstance(targs, np.ndarray)
         metrics = MetricCollection.create_from_model(model)
         results = metrics.compute(targs, preds)
         assert isinstance(results, dict)
