@@ -442,6 +442,31 @@ class TestDatabaseConnectionDataSource:
         pd.testing.assert_frame_equal(dataset.data, expected_output)
         assert dataset.multi_table
 
+    @unit_test
+    def test_database_multi_table_input_table_name(
+        self,
+        db_session: sqlalchemy.engine.base.Engine,
+    ) -> None:
+        """Checks DataSource initialises correctly with `DatabaseConnection`.
+
+        Multi-table database connection, load single table.
+        """
+        db_conn = DatabaseConnection(
+            db_session, table_names=["dummy_data", "dummy_data_2"]
+        )
+
+        dataset = DataSource(db_conn, seed=420)
+
+        # Test when load_data is called without query
+        # DataSource has no data attribute
+        dataset.load_data()
+        assert not hasattr(dataset, "data")
+        # Test when load_data is called WITH query
+        # DataSource has no data attribute
+        table_name = "dummy_data"
+        dataset.load_data(table_name=table_name)
+        assert hasattr(dataset, "data")
+
     @unit_test
     def test_mock_database_multi_table_input(
         self, mock_engine: Mock, mocker: MockerFixture
