@@ -177,7 +177,9 @@ class TestSchemaSerialization:
             ds.loader, "get_dtypes", return_value=create_dataset().dtypes.to_dict()
         )
         mocker.patch.object(
-            ds, "get_values", side_effect=lambda x, y: create_dataset()[x].unique()
+            ds,
+            "get_values",
+            side_effect=lambda x, y: {col: create_dataset()[col].unique() for col in x},
         )
         schema.add_datasource_tables(ds)
         assert schema.table_names == ["dummy_data", "dummy_data_2"]
@@ -996,7 +998,9 @@ class TestBitfountSchema:
             ds.loader, "get_dtypes", return_value=create_dataset().dtypes.to_dict()
         )
         mocker.patch.object(
-            ds, "get_values", side_effect=lambda x, y: create_dataset()[x].unique()
+            ds,
+            "get_values",
+            side_effect=lambda x, y: {col: create_dataset()[col].unique() for col in x},
         )
 
         schema.add_datasource_tables(ds)
@@ -1024,7 +1028,9 @@ class TestBitfountSchema:
             ds.loader, "get_dtypes", return_value=create_dataset().dtypes.to_dict()
         )
         mocker.patch.object(
-            ds, "get_values", side_effect=lambda x, y: create_dataset()[x].unique()
+            ds,
+            "get_values",
+            side_effect=lambda x, y: {col: create_dataset()[col].unique() for col in x},
         )
 
         schema.add_datasource_tables(ds, table_name="this_should_be_ignored")
