@@ -1,7 +1,7 @@
 """Tests for the sql query algorithm."""
 import logging
 from typing import TYPE_CHECKING
-from unittest.mock import create_autospec
+from unittest.mock import Mock, create_autospec
 
 from _pytest.logging import LogCaptureFixture
 import numpy as np
@@ -24,6 +24,7 @@ from bitfount.federated.algorithms.private_sql_query import (
     _ModellerSide,
     _WorkerSide,
 )
+from bitfount.federated.modeller import Modeller
 from bitfount.federated.privacy.differential import DPPodConfig
 from bitfount.hub import BitfountHub
 from tests.utils.helper import TABLE_NAME, create_datasource, create_schema, unit_test
@@ -502,6 +503,25 @@ class TestPrivateSqlQuery:
         assert "Requested DP max epsilon (3.0) exceeds maximum" in caplog.text
         assert "Requested DP target delta (0.1) exceeds maximum" in caplog.text
 
+    @unit_test
+    def test_private_sql_execute(
+        self, mock_bitfount_session: Mock, mocker: MockerFixture, column_ranges: dict
+    ) -> None:
+        """Test execute syntactic sugar."""
+        query = PrivateSqlQuery(
+            query="SELECT * FROM df.df",
+            epsilon=0.1,
+            delta=0.00001,
+            column_ranges=column_ranges,
+        )
+        pod_identifiers = ["username/pod-id"]
+
+        mock_modeller_run_method = mocker.patch.object(Modeller, "run")
+        query.execute(pod_identifiers=pod_identifiers)
+        mock_modeller_run_method.assert_called_once_with(
+            pod_identifiers=pod_identifiers
+        )
+
 
 # Static tests for algorithm-protocol compatibility
 if TYPE_CHECKING:
