@@ -18,6 +18,7 @@ from bitfount.data.datasource import DataSource
 from bitfount.data.schema import BitfountSchema
 from bitfount.data.utils import DatabaseConnection
 from bitfount.federated.authorisation_checkers import IdentityVerificationMethod
+from bitfount.federated.exceptions import PodNameError
 from bitfount.federated.pod import Pod, PodRegistrationError
 from bitfount.federated.pod_keys_setup import PodKeys
 from bitfount.federated.pod_response_message import _PodResponseMessage
@@ -72,7 +73,7 @@ class TestPod:
     @fixture
     def pod_name(self) -> str:
         """Pod name."""
-        return "somePodName"
+        return "some-pod-name"
 
     @fixture
     def mock_dataframe(self) -> Mock:
@@ -281,6 +282,39 @@ class TestPod:
         """An AES key."""
         return b"someAesKey"
 
+    @pytest.mark.parametrize(
+        "invalid_pod_name",
+        ["Invalid", "INVALID", "invalid-", "-invalid", "invalid--invalid"],
+    )
+    def test_invalid_pod_name(
+        self,
+        approved_pods: List[str],
+        mock_access_manager: Mock,
+        mock_bitfount_hub: Mock,
+        mock_dataframe: Mock,
+        mock_message_service_config: DataclassMock,
+        mock_pod_data_config: DataclassMock,
+        mock_pod_details_config: DataclassMock,
+        mock_pod_keys: DataclassMock,
+        mocker: MockerFixture,
+        username: str,
+        invalid_pod_name: str,
+    ) -> None:
+        """Test PodNameError raised for invalid pod name."""
+        with pytest.raises(PodNameError):
+            Pod(
+                name=invalid_pod_name,
+                data=mock_dataframe,
+                username=username,
+                data_config=mock_pod_data_config,
+                pod_details_config=mock_pod_details_config,
+                bitfounthub=mock_bitfount_hub,
+                ms_config=mock_message_service_config,
+                access_manager=mock_access_manager,
+                pod_keys=mock_pod_keys,
+                approved_pods=approved_pods,
+            )
+
     async def test__check_for_unapproved_pods_without_secure_aggregation(
         self, pod: Pod, pod_identifier: str
     ) -> None:
