@@ -102,7 +102,9 @@ class TestLogisticRegression:
     ) -> None:
         """Test LogisticRegression fit() and get_results() methods."""
         log_reg = LogisticRegressionClassifier(
-            datastructure=datastructure, schema=BitfountSchema(), verbose=0
+            datastructure=datastructure,
+            schema=create_schema(classification=True),
+            verbose=0,
         )
         log_reg.fit(datasource)
         assert_results(model=log_reg)
@@ -115,7 +117,9 @@ class TestLogisticRegression:
             target=["TARGET"], image_cols=["img"], table=TABLE_NAME
         )
         model = LogisticRegressionClassifier(
-            datastructure=datastructure, schema=BitfountSchema(), verbose=0
+            datastructure=datastructure,
+            schema=create_schema(classification=True),
+            verbose=0,
         )
         with pytest.raises(ValueError):
             model.fit(datasource)
@@ -124,7 +128,9 @@ class TestLogisticRegression:
     def test_evaluate_no_test_dl_error(self, datastructure: DataStructure) -> None:
         """Tests that evaluate raises error with no test_dl."""
         model = LogisticRegressionClassifier(
-            datastructure=datastructure, schema=BitfountSchema(), verbose=-1
+            datastructure=datastructure,
+            schema=create_schema(classification=True),
+            verbose=-1,
         )
         with pytest.raises(ValueError):
             model.evaluate()
@@ -135,14 +141,16 @@ class TestLogisticRegression:
     ) -> None:
         """Test serialize() and deserialize() methods."""
         log_reg = LogisticRegressionClassifier(
-            datastructure=datastructure, schema=BitfountSchema(), verbose=0
+            datastructure=datastructure,
+            schema=create_schema(classification=True),
+            verbose=0,
         )
         log_reg.fit(datasource)
         log_reg.serialize(tmp_path / SERIALIZED_MODEL_NAME)
         assert os.path.exists(tmp_path / SERIALIZED_MODEL_NAME)
 
         log_reg = LogisticRegressionClassifier(
-            datastructure=datastructure, schema=BitfountSchema()
+            datastructure=datastructure, schema=create_schema(classification=True)
         )
         log_reg.fit(datasource)
         log_reg.deserialize(tmp_path / SERIALIZED_MODEL_NAME)
