@@ -30,6 +30,7 @@ import pytest
 from bitfount.data.datasource import DataSource
 from bitfount.data.datastructure import DataStructure
 from bitfount.data.schema import BitfountSchema
+from bitfount.data.types import SchemaOverrideMapping
 from bitfount.federated.aggregators.aggregator import Aggregator
 from bitfount.federated.aggregators.base import _BaseAggregatorFactory
 from bitfount.federated.aggregators.secure import SecureAggregator
@@ -288,6 +289,51 @@ def create_datastructure(
     )
 
 
+def create_query_datastructure(
+    multilabel: bool = False,
+    multihead: bool = False,
+    multihead_size: int = 1,
+    loss_weights: bool = False,
+) -> DataStructure:
+    """This method creates and returns a DataStructure object."""
+    target: Union[str, List[str]] = "target"
+    loss_weights_col = None
+    multihead_col = None
+    query = """SELECT dd1."A" as a1, dd1."D" as d1, dd1."M" as m1,
+                dd1."N" as n1, dd1."TARGET" as target,
+                dd2."A" as a2, dd2."D" as d2, dd2."M" as m2, dd2."N" as n2
+                FROM dummy_data dd1, dummy_data_2 dd2
+                WHERE dd1.index = dd2.index
+    """
+    feature_override: SchemaOverrideMapping = {
+        "continuous": ["a1", "d1", "a2", "d2"],
+        "categorical": [
+            {"m1": {"True": 1, "False": 0}},
+            {"n1": {"True": 1, "False": 0}},
+            {"m2": {"True": 1, "False": 0}},
+            {"n2": {"True": 1, "False": 0}},
+            {"target": {"0": 0, "1": 1}},
+        ],
+    }
+    if loss_weights:
+        loss_weights_col = "weights"
+
+    if multihead:
+        multihead_col = "category"
+
+    if multilabel:
+        target = [cast(str, target), "TARGET_2"]
+
+    return DataStructure(
+        query=query,
+        schema_types_override=feature_override,
+        target=target,
+        multihead_col=multihead_col,
+        multihead_size=multihead_size,
+        loss_weights_col=loss_weights_col,
+    )
+
+
 def create_datasource(
     classification: bool,
     multilabel: bool = False,
@@ -589,15 +635,16 @@ def assert_results(
     This method takes a fitted model, evaluates it and performs some checks
     on the output.
     """
-    preds: np.ndarray
-    target: np.ndarray
     if test_preds is None or test_target is None:
-        preds, target = model.evaluate()
+        preds, targs = model.evaluate()
+        # TODO: [BIT-1604] Remove these assert statements once they become superfluous.
+        assert isinstance(preds, np.ndarray)
+        assert isinstance(targs, np.ndarray)
     else:
         preds = test_preds
-        target = np.asarray(test_target)
+        targs = np.asarray(test_target)
     metrics = MetricCollection.create_from_model(model)
-    results = metrics.compute(target, preds)
+    results = metrics.compute(targs, preds)
     assert results is not None
     assert isinstance(results, dict)
     if isinstance(model, ClassifierMixIn):
